#include "foursitejump.h"
#include "coordinates.h"

#include <random>

namespace motions {
    FourSiteTetrahedron::FourSiteTetrahedron(const double delta, const double eta, std::mt19937_64& rng) :
        BaseMotion(std::string{"FourSiteTetrahedral"}, delta, eta, rng) {}

    FourSiteTetrahedron::FourSiteTetrahedron(std::mt19937_64& rng) : BaseMotion(std::string{"FourSiteTetrahedral"}, rng) {}

    void FourSiteTetrahedron::initialize() {
        const auto pos = draw_position();
        m_corners[0] = omega_q(pos);

        const double alpha = 2. * M_PI * m_uni_dist(m_rng);

        for (int i = 1; i<4; i++) {
            auto corner_pos = coordinates::rotate(pos, m_beta, alpha + (i-1) * 2*M_PI/3.);
            m_corners[i] = omega_q(corner_pos);
        }
        m_initial_omega = FourSiteTetrahedron::jump();
    }

    double FourSiteTetrahedron::jump() {
        m_corner_idx += m_chooser(m_rng);
        m_corner_idx %= 4;

        return m_corners[m_corner_idx];
    }

    std::string FourSiteTetrahedron::toString() const {
        return {"FourSiteTetrahedral"};
    }
}